with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions,
  terms;

procedure antenna is

  type elem1_rec is
    record
      glob : terms.imat1 (1 .. 2);
    end record;

  type elem1_vec is array (integer range <>) of elem1_rec;

  type elem1_ptr is access elem1_vec;

  type elem2_rec is
    record
      glob : terms.imat1 (1 .. 3);
    end record;

  type elem2_vec is array (integer range <>) of elem2_rec;

  type elem2_ptr is access elem2_vec;

  ns,ni,nm,nn,ne        : integer;
  radius0,radius,height : long_float;
  stiffness,poisson     : long_float;
  area0,young0          : long_float;
  area,young,tension    : long_float;
  xo,yo,zo              : terms.fmat1_ptr;
  cost,sint             : terms.fmat1_ptr;
  elem1                 : elem1_ptr;
  elem2                 : elem2_ptr;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  function node (i,j : in integer) return integer is

  begin
    if i = 0 and j = 0 then
      return 1;
    else
      return i * (i - 1) / 2 * ns + j + 2;
    end if;
  end node;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io,
      ada.numerics,
      ada.numerics.long_elementary_functions;

    e,glob,k : integer;
    angle    : long_float;
    io       : file_type;

  begin
    -- A regular polygon with ns sides is inscribed in a circumference
    -- The radius is divided in ni equal parts

    open(io, in_file, "antenna.txt");

    -- ns, ni, radius0, radius, height
    scan(io);
    get(io,ns);
    get(io,ni);
    get(io,radius0);
    get(io,radius);
    get(io,height);
    skip_line(io);

    -- stiffness, poisson
    scan(io);
    get(io,stiffness);
    get(io,poisson);
    skip_line(io);

    -- area0, young0
    scan(io);
    get(io,area0);
    get(io,young0);
    skip_line(io);

    -- area, young, tension
    scan(io);
    get(io,area);
    get(io,young);
    get(io,tension);
    skip_line(io);

    close(io);

    -- number of membrane nodes
    nm := (ni + 1) * ni / 2 * ns + 1;

    -- number of nodes
    nn := nm + ns;

    -- cost, sint
    cost := new terms.fmat1 (0 .. (ns - 1));
    sint := new terms.fmat1 (0 .. (ns - 1));
    angle := 2.0 * pi / long_float(ns);
    for k in 0 .. ns - 1 loop
      cost(k) := cos(angle * long_float(k));
      sint(k) := sin(angle * long_float(k));
    end loop;

    -- xo, yo, zo
    xo := new terms.fmat1 (1 .. nn);
    yo := new terms.fmat1 (1 .. nn);
    zo := new terms.fmat1 (1 .. nn);

    -- node coordinates
    glob := node(0,0);
    xo(glob) := 0.0;
    yo(glob) := 0.0;
    zo(glob) := height;

    for i in 1 .. ni loop
      for j in 0 .. i * ns - 1 loop
        glob := node(i,j);
        k := j / i;
        xo(glob) := radius * (long_float(i - j + k * i) * cost(k) + long_float(j - k * i) * cost((k + 1) rem ns)) / long_float(ni);
        yo(glob) := radius * (long_float(i - j + k * i) * sint(k) + long_float(j - k * i) * sint((k + 1) rem ns)) / long_float(ni);
        zo(glob) := height;
      end loop;
    end loop;

    for k in 0 .. ns - 1 loop
      glob := nm + k + 1;
      xo(glob) := radius0 * cost(k);
      yo(glob) := radius0 * sint(k);
      zo(glob) := 0.0;
    end loop;

    -- elements 1
    ne := ns + ns;
    elem1 := new elem1_vec (1 .. ne);

    for k in 0 .. ns - 1 loop
      elem1(k + 1).glob(1) := nm + k + 1;
      elem1(k + 1).glob(2) := node(ni,k * ni);
    end loop;

    for k in 0 .. ns - 2 loop
      elem1(ns + k + 1).glob(1) := nm + k + 1;
      elem1(ns + k + 1).glob(2) := node(ni,(k + 1) * ni);
    end loop;
    elem1(ns + ns).glob(1) := nm + ns;
    elem1(ns + ns).glob(2) := node(ni,0);

    -- elements 2
    ne := ns * ni * ni;
    elem2 := new elem2_vec (1 .. ne);

    e := 0;

    for j in 0 .. ns - 2 loop
      e := e + 1;
      elem2(e).glob(1) := node(0,0);
      elem2(e).glob(2) := node(1,j);
      elem2(e).glob(3) := node(1,j + 1);
    end loop;

    e := e + 1;
    elem2(e).glob(1) := node(0,0);
    elem2(e).glob(2) := node(1,ns - 1);
    elem2(e).glob(3) := node(1,0);

    for i in 2 .. ni loop
      for j in 0 .. i * ns - 2 loop
        k := j / i;
        e := e + 1;
        elem2(e).glob(1) := node(i - 1,j - k);
        elem2(e).glob(2) := node(i,j);
        elem2(e).glob(3) := node(i,j + 1);
      end loop;

      e := e + 1;
      elem2(e).glob(1) := node(i - 1,0);
      elem2(e).glob(2) := node(i,i * ns - 1);
      elem2(e).glob(3) := node(i,0);
    end loop;

    for i in 1 .. ni - 1 loop
      for j in 0 .. i * ns - 2 loop
        k := j / i;
        e := e + 1;
        elem2(e).glob(1) := node(i + 1,j + 1 + k);
        elem2(e).glob(2) := node(i,j + 1);
        elem2(e).glob(3) := node(i,j);
      end loop;

      e := e + 1;
      elem2(e).glob(1) := node(i + 1,i * ns + ns - 1);
      elem2(e).glob(2) := node(i,0);
      elem2(e).glob(3) := node(i,i * ns - 1);
    end loop;
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist : integer;
    lstol,mntol,mnref      : long_float;
    io                     : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    mntol := 1.0E-06;
    mnref := 1.0;
    maxminor := 30 * nn;
    maxlist := 3;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls             mntol             mnref  maxminor   maxlist");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,mntol,2,6,3);
    put(io,"     ");
    put(io,mnref,2,6,3);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nn,5);
    new_line(io);

    new_line(io);
    for glob in 1 .. nn loop
      put(io,glob,5);
      put(io,"     ");
      put(io,xo(glob),2,6,3);
      put(io,"     ");
      put(io,yo(glob),2,6,3);
      put(io,"     ");
      put(io,zo(glob),2,6,3);
      new_line(io);
    end loop;

    -- elements 1
    new_line(io);
    new_line(io);
    put(io," elem      node      node              area             young");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ns + ns,5);
    new_line(io);

    new_line(io);
    for e in 1 .. ns loop
      put(io,e,5);
      for loc in 1 .. 2 loop
        put(io,"     ");
        put(io,elem1(e).glob(loc),5);
      end loop;
      put(io,"     ");
      put(io,area0,2,6,3);
      put(io,"     ");
      put(io,young0,2,6,3);
      new_line(io);
    end loop;

    for e in ns + 1 .. ns + ns loop
      put(io,e,5);
      for loc in 1 .. 2 loop
        put(io,"     ");
        put(io,elem1(e).glob(loc),5);
      end loop;
      put(io,"     ");
      put(io,area,2,6,3);
      put(io,"     ");
      put(io,young,2,6,3);
      new_line(io);
    end loop;

    -- elements 2
    new_line(io);
    new_line(io);
    put(io," elem      node      node      node         stiffness           poisson");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,elem2'last,5);
    new_line(io);

    new_line(io);
    for e in elem2'range loop
      put(io,e,5);
      for loc in 1 .. 3 loop
        put(io,"     ");
        put(io,elem2(e).glob(loc),5);
      end loop;
      put(io,"     ");
      put(io,stiffness,2,6,3);
      put(io,"     ");
      put(io,poisson,2,6,3);
      new_line(io);
    end loop;

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,3 * ns,5);
    new_line(io);

    new_line(io);
    for glob in nm + 1 .. nn loop
      put(io,glob,5);
      put(io,"         1      0.000000E+00");
      new_line(io);

      put(io,glob,5);
      put(io,"         2      0.000000E+00");
      new_line(io);

      put(io,glob,5);
      put(io,"         3      0.000000E+00");
      new_line(io);
    end loop;

    -- set cable element state
    new_line(io);
    new_line(io);
    put(io," elem     state           tension");
    new_line(io);

    put(io,":");
    new_line(io);
    put(io,ns,5);
    new_line(io);

    new_line(io);
    for e in ns + 1 .. ns + ns loop
      put(io,e,5);
      put(io,"         2");
      put(io,"     ");
      put(io,tension,2,6,3);
      new_line(io);
    end loop;

    -- loading
    new_line(io);
    new_line(io);
    put(io," node      axis             force");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    glob : integer;
    io   : file_type;

  begin
    create(io, out_file, "antenna.scr");

    -- set running object snap mode to NONE
    put(io,"osmode 0");
    new_line(io);

    -- elements 2
    put(io,"layer make Elem2 ");
    new_line(io);

    for e in elem2'range loop
      put(io,"3dface ");
      for loc in 1 .. 3 loop
        glob := elem2(e).glob(loc);
        put(io,xo(glob),1,8,3);
        put(io,",");
        put(io,yo(glob),1,8,3);
        put(io,",");
        put(io,zo(glob),1,8,3);
        put(io," ");
      end loop;
      put(io," ");
      new_line(io);
    end loop;

    -- elements 1
    put(io,"layer make Elem1 ");
    new_line(io);

    for e in 1 .. ns loop
      put(io,"3dpoly ");
      for loc in 1 .. 2 loop
        glob := elem1(e).glob(loc);
        put(io,xo(glob),1,8,3);
        put(io,",");
        put(io,yo(glob),1,8,3);
        put(io,",");
        put(io,zo(glob),1,8,3);
        put(io," ");
      end loop;
      new_line(io);
    end loop;

    put(io,"-color");
    new_line(io);
    put(io,"green");
    new_line(io);

    for e in ns + 1 .. ns + ns loop
      put(io,"3dpoly ");
      for loc in 1 .. 2 loop
        glob := elem1(e).glob(loc);
        put(io,xo(glob),1,8,3);
        put(io,",");
        put(io,yo(glob),1,8,3);
        put(io,",");
        put(io,zo(glob),1,8,3);
        put(io," ");
      end loop;
      new_line(io);
    end loop;

    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    -- autocad
    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  input;
  autocad;
end antenna;
