with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions,
  terms;

procedure prism2 is

  type elem1_rec is
    record
      glob : terms.imat1 (1 .. 2);
    end record;

  type elem1_vec is array (integer range <>) of elem1_rec;

  type elem1_ptr is access elem1_vec;

  type elem2_rec is
    record
      glob : terms.imat1 (1 .. 3);
    end record;

  type elem2_vec is array (integer range <>) of elem2_rec;

  type elem2_ptr is access elem2_vec;

  nn,state          : integer;
  area,young,value  : long_float;
  stiffness,poisson : long_float;
  xo,yo,zo          : terms.fmat1_ptr;
  elem1             : elem1_ptr;
  elem2             : elem2_ptr;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io,
      ada.numerics,
      ada.numerics.long_elementary_functions;

    nc,np,ni,nj,glob,ne : integer;
    angle               : long_float;
    rho1,rho2,rho3      : long_float;
    radius,height       : terms.fmat1_ptr;
    io                  : file_type;

    procedure mesh (ni,nj                   : in integer;
                    rho1,rho2,rho3          : in long_float;
                    glob1,glob2,glob3,glob4 : in integer) is

        --            rho3
        --      glob4      glob3
        -- rho2                  rho2
        --      glob1      glob2
        --            rho1

      use
        ada.numerics.long_elementary_functions;

      type vec3 is array (1 .. 3) of long_float;

      type surface_rec is
        record
          o,u,v,w : vec3;
        end record;

      type sphere_rec is
        record
          radius : long_float;
          c,u,p  : vec3;
        end record;

      i,j,mi,mj,glob          : integer;
      norm,dist1,dist2,alpha  : long_float;
      node                    : terms.imat2_ptr;
      a,b,pi1,pi2             : vec3;
      surface                 : surface_rec;
      sphere1,sphere2,sphere3 : sphere_rec;

      procedure getsphere (rho         : in long_float;
                           glob1,glob2 : in integer;
                           w           : in vec3;
                           sphere      : out sphere_rec) is

        norm : long_float;
        v    : vec3;

      begin
        sphere.p(1) := (xo(glob1) + xo(glob2)) / 2.0;
        sphere.p(2) := (yo(glob1) + yo(glob2)) / 2.0;
        sphere.p(3) := (zo(glob1) + zo(glob2)) / 2.0;

        v(1) := xo(glob2) - xo(glob1);
        v(2) := yo(glob2) - yo(glob1);
        v(3) := zo(glob2) - zo(glob1);
        norm := sqrt(v(1) ** 2 + v(2) ** 2 + v(3) ** 2);

        sphere.radius := (rho ** 2 + 0.25) * norm / rho / 2.0;

        sphere.u(1) := v(2) * w(3) - v(3) * w(2);
        sphere.u(2) := v(3) * w(1) - v(1) * w(3);
        sphere.u(3) := v(1) * w(2) - v(2) * w(1);
        norm := sqrt(sphere.u(1) ** 2 + sphere.u(2) ** 2 + sphere.u(3) ** 2);
        sphere.u(1) := sphere.u(1) / norm;
        sphere.u(2) := sphere.u(2) / norm;
        sphere.u(3) := sphere.u(3) / norm;

        norm := sphere.radius - norm * rho;
        sphere.c(1) := norm * sphere.u(1) + sphere.p(1);
        sphere.c(2) := norm * sphere.u(2) + sphere.p(2);
        sphere.c(3) := norm * sphere.u(3) + sphere.p(3);
      end getsphere;

      procedure spherecross1 (a,b    : in vec3;
                              sphere : in sphere_rec;
                              p      : out vec3) is

        sum1,sum2,alpha : long_float;

      begin
        -- intersection of line with sphere
        sum1 := 0.0;
        sum1 := sum1 + (b(1) - sphere.c(1)) * a(1);
        sum1 := sum1 + (b(2) - sphere.c(2)) * a(2);
        sum1 := sum1 + (b(3) - sphere.c(3)) * a(3);

        sum2 := 0.0;
        sum2 := sum2 + (b(1) - sphere.c(1)) ** 2;
        sum2 := sum2 + (b(2) - sphere.c(2)) ** 2;
        sum2 := sum2 + (b(3) - sphere.c(3)) ** 2;

        alpha := -sum1 - sqrt(sum1 ** 2 - sum2 + sphere.radius ** 2);
        p(1) := alpha * a(1) + b(1);
        p(2) := alpha * a(2) + b(2);
        p(3) := alpha * a(3) + b(3);
      end spherecross1;

      procedure spherecross2 (a,b    : in vec3;
                              sphere : in sphere_rec;
                              p      : out vec3) is

        sum1,sum2,alpha : long_float;

      begin
        -- intersection of line with sphere
        sum1 := 0.0;
        sum1 := sum1 + (b(1) - sphere.c(1)) * a(1);
        sum1 := sum1 + (b(2) - sphere.c(2)) * a(2);
        sum1 := sum1 + (b(3) - sphere.c(3)) * a(3);

        sum2 := 0.0;
        sum2 := sum2 + (b(1) - sphere.c(1)) ** 2;
        sum2 := sum2 + (b(2) - sphere.c(2)) ** 2;
        sum2 := sum2 + (b(3) - sphere.c(3)) ** 2;

        alpha := -sum1 + sqrt(sum1 ** 2 - sum2 + sphere.radius ** 2);
        p(1) := alpha * a(1) + b(1);
        p(2) := alpha * a(2) + b(2);
        p(3) := alpha * a(3) + b(3);
      end spherecross2;

      procedure planecross (a,b,w,x0 : in vec3;
                            p        : out vec3) is

        sum1,sum2,alpha : long_float;

      begin
        -- intersection of line with plane
        sum1 := 0.0;
        sum1 := sum1 + w(1) * (x0(1) - b(1));
        sum1 := sum1 + w(2) * (x0(2) - b(2));
        sum1 := sum1 + w(3) * (x0(3) - b(3));

        sum2 := 0.0;
        sum2 := sum2 + w(1) * a(1);
        sum2 := sum2 + w(2) * a(2);
        sum2 := sum2 + w(3) * a(3);

        alpha := sum1 / sum2;
        p(1) := alpha * a(1) + b(1);
        p(2) := alpha * a(2) + b(2);
        p(3) := alpha * a(3) + b(3);
      end planecross;

    begin
      -- surface
      surface.o(1) := (xo(glob1) + xo(glob2) + xo(glob3) + xo(glob4)) / 4.0;
      surface.o(2) := (yo(glob1) + yo(glob2) + yo(glob3) + yo(glob4)) / 4.0;
      surface.o(3) := (zo(glob1) + zo(glob2) + zo(glob3) + zo(glob4)) / 4.0;

      surface.u(1) := (xo(glob2) + xo(glob3)) / 2.0 - (xo(glob4) + xo(glob1)) / 2.0;
      surface.u(2) := (yo(glob2) + yo(glob3)) / 2.0 - (yo(glob4) + yo(glob1)) / 2.0;
      surface.u(3) := (zo(glob2) + zo(glob3)) / 2.0 - (zo(glob4) + zo(glob1)) / 2.0;

      surface.v(1) := (xo(glob3) + xo(glob4)) / 2.0 - (xo(glob1) + xo(glob2)) / 2.0;
      surface.v(2) := (yo(glob3) + yo(glob4)) / 2.0 - (yo(glob1) + yo(glob2)) / 2.0;
      surface.v(3) := (zo(glob3) + zo(glob4)) / 2.0 - (zo(glob1) + zo(glob2)) / 2.0;

      surface.w(1) := surface.u(2) * surface.v(3) - surface.u(3) * surface.v(2);
      surface.w(2) := surface.u(3) * surface.v(1) - surface.u(1) * surface.v(3);
      surface.w(3) := surface.u(1) * surface.v(2) - surface.u(2) * surface.v(1);

      norm := sqrt(surface.u(1) ** 2 + surface.u(2) ** 2 + surface.u(3) ** 2);
      surface.u(1) := surface.u(1) / norm;
      surface.u(2) := surface.u(2) / norm;
      surface.u(3) := surface.u(3) / norm;

      norm := sqrt(surface.v(1) ** 2 + surface.v(2) ** 2 + surface.v(3) ** 2);
      surface.v(1) := surface.v(1) / norm;
      surface.v(2) := surface.v(2) / norm;
      surface.v(3) := surface.v(3) / norm;

      norm := sqrt(surface.w(1) ** 2 + surface.w(2) ** 2 + surface.w(3) ** 2);
      surface.w(1) := surface.w(1) / norm;
      surface.w(2) := surface.w(2) / norm;
      surface.w(3) := surface.w(3) / norm;

      -- sphere
      getsphere(rho1,glob1,glob2,surface.w,sphere1);
      getsphere(rho2,glob2,glob3,surface.w,sphere2);
      getsphere(rho3,glob3,glob4,surface.w,sphere3);

      -- node
      node := new terms.imat2 (-ni .. ni, -nj .. nj);

      for j in -nj .. nj loop
        for i in -ni .. ni loop
          node(i,j) := 0;
        end loop;
      end loop;

      node(-ni,-nj) := glob1;
      node( ni,-nj) := glob2;
      node( ni, nj) := glob3;
      node(-ni, nj) := glob4;

      i := -ni + 2;
      while i <= ni - 2 loop
        nn := nn + 1;
        node(i,-nj) := nn;
        i := i + 2;
      end loop;

      for j in -nj + 1 .. nj - 1 loop
        mi := ni - abs(j) mod 2;
        i := -mi;
        while i <= mi loop
          nn := nn + 1;
          node(i,j) := nn;
          i := i + 2;
        end loop;
      end loop;

      i := -ni + 2;
      while i <= ni - 2 loop
        nn := nn + 1;
        node(i,nj) := nn;
        i := i + 2;
      end loop;

      -- node coordinates
      for i in 0 .. ni - 1 loop
        alpha := long_float(i) / long_float(ni);

        pi1(1) := ((xo(glob2) + xo(glob1)) + (xo(glob2) - xo(glob1)) * alpha) / 2.0;
        pi1(2) := ((yo(glob2) + yo(glob1)) + (yo(glob2) - yo(glob1)) * alpha) / 2.0;
        pi1(3) := ((zo(glob2) + zo(glob1)) + (zo(glob2) - zo(glob1)) * alpha) / 2.0;

        pi2(1) := ((xo(glob3) + xo(glob4)) + (xo(glob3) - xo(glob4)) * alpha) / 2.0;
        pi2(2) := ((yo(glob3) + yo(glob4)) + (yo(glob3) - yo(glob4)) * alpha) / 2.0;
        pi2(3) := ((zo(glob3) + zo(glob4)) + (zo(glob3) - zo(glob4)) * alpha) / 2.0;

        a(1) := pi2(1) - pi1(1);
        a(2) := pi2(2) - pi1(2);
        a(3) := pi2(3) - pi1(3);
        norm := sqrt(a(1) ** 2 + a(2) ** 2 + a(3) ** 2);
        a(1) := a(1) / norm;
        a(2) := a(2) / norm;
        a(3) := a(3) / norm;

        b(1) := (pi1(1) + pi2(1)) / 2.0;
        b(2) := (pi1(2) + pi2(2)) / 2.0;
        b(3) := (pi1(3) + pi2(3)) / 2.0;

        -- intersection with sphere
        spherecross2(a,b,sphere1,pi1);

        -- intersection with sphere
        spherecross1(a,b,sphere3,pi2);

        mj := nj - abs(i) mod 2;
        j := -mj;
        while j <= mj loop
          alpha := (long_float(j) / long_float(nj) + 1.0) / 2.0;
          glob := node(i,j);
          xo(glob) := (1.0 - alpha) * pi1(1) + alpha * pi2(1);
          yo(glob) := (1.0 - alpha) * pi1(2) + alpha * pi2(2);
          zo(glob) := (1.0 - alpha) * pi1(3) + alpha * pi2(3);
          j := j + 2;
        end loop;
      end loop;

      j := -nj + 2;
      while j <= nj - 2 loop
        alpha := (long_float(j) / long_float(nj) + 1.0) / 2.0;
        glob := node(ni,j);
        xo(glob) := (1.0 - alpha) * xo(glob2) + alpha * xo(glob3);
        yo(glob) := (1.0 - alpha) * yo(glob2) + alpha * yo(glob3);
        zo(glob) := (1.0 - alpha) * zo(glob2) + alpha * zo(glob3);
        j := j + 2;
      end loop;

      -- node coordinates
      i := ni - 2;
      while i > 0 loop
        glob := node(i,-nj);

        b(1) := xo(glob);
        b(2) := yo(glob);
        b(3) := zo(glob);

        -- intersection with middle plane
        planecross(surface.u,b,surface.u,surface.o,pi1);

        dist1 := 0.0;
        dist1 := dist1 + (pi1(1) - xo(glob)) ** 2;
        dist1 := dist1 + (pi1(2) - yo(glob)) ** 2;
        dist1 := dist1 + (pi1(3) - zo(glob)) ** 2;
        dist1 := sqrt(dist1);

        -- intersection with right plane
        planecross(surface.u,b,sphere2.u,sphere2.p,pi2);

        dist2 := 0.0;
        dist2 := dist2 + (pi2(1) - xo(glob)) ** 2;
        dist2 := dist2 + (pi2(2) - yo(glob)) ** 2;
        dist2 := dist2 + (pi2(3) - zo(glob)) ** 2;
        dist2 := sqrt(dist2);

        -- alpha
        alpha := dist1 / (dist1 + dist2);

        -- intersection with sphere
        spherecross1(surface.u,b,sphere2,pi2);

        b(1) := (1.0 - alpha) * pi1(1) + alpha * pi2(1);
        b(2) := (1.0 - alpha) * pi1(2) + alpha * pi2(2);
        b(3) := (1.0 - alpha) * pi1(3) + alpha * pi2(3);

        -- intersection with sphere
        spherecross2(surface.v,b,sphere1,pi1);

        xo(glob) := pi1(1);
        yo(glob) := pi1(2);
        zo(glob) := pi1(3);

        i := i - 2;
      end loop;

      for j in -nj + 1 .. nj - 1 loop
        mi := ni - abs(j) mod 2;
        i := mi;
        while i > 0 loop
          glob := node(i,j);

          b(1) := xo(glob);
          b(2) := yo(glob);
          b(3) := zo(glob);

          -- intersection with middle plane
          planecross(surface.u,b,surface.u,surface.o,pi1);

          dist1 := 0.0;
          dist1 := dist1 + (pi1(1) - xo(glob)) ** 2;
          dist1 := dist1 + (pi1(2) - yo(glob)) ** 2;
          dist1 := dist1 + (pi1(3) - zo(glob)) ** 2;
          dist1 := sqrt(dist1);

          -- intersection with right plane
          planecross(surface.u,b,sphere2.u,sphere2.p,pi2);

          dist2 := 0.0;
          dist2 := dist2 + (pi2(1) - xo(glob)) ** 2;
          dist2 := dist2 + (pi2(2) - yo(glob)) ** 2;
          dist2 := dist2 + (pi2(3) - zo(glob)) ** 2;
          dist2 := sqrt(dist2);

          -- alpha
          alpha := dist1 / (dist1 + dist2);

          -- intersection with sphere
          spherecross1(surface.u,b,sphere2,pi2);

          xo(glob) := (1.0 - alpha) * pi1(1) + alpha * pi2(1);
          yo(glob) := (1.0 - alpha) * pi1(2) + alpha * pi2(2);
          zo(glob) := (1.0 - alpha) * pi1(3) + alpha * pi2(3);

          i := i - 2;
        end loop;
      end loop;

      i := ni - 2;
      while i > 0 loop
        glob := node(i,nj);

        b(1) := xo(glob);
        b(2) := yo(glob);
        b(3) := zo(glob);

        -- intersection with middle plane
        planecross(surface.u,b,surface.u,surface.o,pi1);

        dist1 := 0.0;
        dist1 := dist1 + (pi1(1) - xo(glob)) ** 2;
        dist1 := dist1 + (pi1(2) - yo(glob)) ** 2;
        dist1 := dist1 + (pi1(3) - zo(glob)) ** 2;
        dist1 := sqrt(dist1);

        -- intersection with right plane
        planecross(surface.u,b,sphere2.u,sphere2.p,pi2);

        dist2 := 0.0;
        dist2 := dist2 + (pi2(1) - xo(glob)) ** 2;
        dist2 := dist2 + (pi2(2) - yo(glob)) ** 2;
        dist2 := dist2 + (pi2(3) - zo(glob)) ** 2;
        dist2 := sqrt(dist2);

        -- alpha
        alpha := dist1 / (dist1 + dist2);

        -- intersection with sphere
        spherecross1(surface.u,b,sphere2,pi2);

        b(1) := (1.0 - alpha) * pi1(1) + alpha * pi2(1);
        b(2) := (1.0 - alpha) * pi1(2) + alpha * pi2(2);
        b(3) := (1.0 - alpha) * pi1(3) + alpha * pi2(3);

        -- intersection with sphere
        spherecross1(surface.v,b,sphere3,pi1);

        xo(glob) := pi1(1);
        yo(glob) := pi1(2);
        zo(glob) := pi1(3);

        i := i - 2;
      end loop;

      -- symmetry
      i := ni - 2;
      while i > 0 loop
        glob := node(i,-nj);

        b(1) := xo(glob);
        b(2) := yo(glob);
        b(3) := zo(glob);

        -- intersection with middle plane
        planecross(surface.u,b,surface.u,surface.o,pi1);

        alpha := 0.0;
        alpha := alpha + (pi1(1) - xo(glob)) ** 2;
        alpha := alpha + (pi1(2) - yo(glob)) ** 2;
        alpha := alpha + (pi1(3) - zo(glob)) ** 2;
        alpha := sqrt(alpha);

        glob := node(-i,-nj);

        xo(glob) := pi1(1) - alpha * surface.u(1);
        yo(glob) := pi1(2) - alpha * surface.u(2);
        zo(glob) := pi1(3) - alpha * surface.u(3);

        i := i - 2;
      end loop;

      for j in -nj + 1 .. nj - 1 loop
        mi := ni - abs(j) mod 2;
        i := mi;
        while i > 0 loop
          glob := node(i,j);

          b(1) := xo(glob);
          b(2) := yo(glob);
          b(3) := zo(glob);

          -- intersection with middle plane
          planecross(surface.u,b,surface.u,surface.o,pi1);

          alpha := 0.0;
          alpha := alpha + (pi1(1) - xo(glob)) ** 2;
          alpha := alpha + (pi1(2) - yo(glob)) ** 2;
          alpha := alpha + (pi1(3) - zo(glob)) ** 2;
          alpha := sqrt(alpha);

          glob := node(-i,j);

          xo(glob) := pi1(1) - alpha * surface.u(1);
          yo(glob) := pi1(2) - alpha * surface.u(2);
          zo(glob) := pi1(3) - alpha * surface.u(3);

          i := i - 2;
        end loop;
      end loop;

      i := ni - 2;
      while i > 0 loop
        glob := node(i,nj);

        b(1) := xo(glob);
        b(2) := yo(glob);
        b(3) := zo(glob);

        -- intersection with middle plane
        planecross(surface.u,b,surface.u,surface.o,pi1);

        alpha := 0.0;
        alpha := alpha + (pi1(1) - xo(glob)) ** 2;
        alpha := alpha + (pi1(2) - yo(glob)) ** 2;
        alpha := alpha + (pi1(3) - zo(glob)) ** 2;
        alpha := sqrt(alpha);

        glob := node(-i,nj);

        xo(glob) := pi1(1) - alpha * surface.u(1);
        yo(glob) := pi1(2) - alpha * surface.u(2);
        zo(glob) := pi1(3) - alpha * surface.u(3);

        i := i - 2;
      end loop;

      -- elements 2
      j := -nj + 1;
      while j <= nj - 1 loop
        i := -ni + 1;
        while i <= ni - 1 loop
          -- (i-1,j+1)       (i+1,j+1)
          --           (i,j)
          -- (i-1,j-1)       (i+1,j-1)

          ne := ne + 1;
          elem2(ne).glob(1) := node(i,j);
          elem2(ne).glob(2) := node(i - 1,j - 1);
          elem2(ne).glob(3) := node(i + 1,j - 1);

          ne := ne + 1;
          elem2(ne).glob(1) := node(i,j);
          elem2(ne).glob(2) := node(i + 1,j - 1);
          elem2(ne).glob(3) := node(i + 1,j + 1);

          ne := ne + 1;
          elem2(ne).glob(1) := node(i,j);
          elem2(ne).glob(2) := node(i + 1,j + 1);
          elem2(ne).glob(3) := node(i - 1,j + 1);

          ne := ne + 1;
          elem2(ne).glob(1) := node(i,j);
          elem2(ne).glob(2) := node(i - 1,j + 1);
          elem2(ne).glob(3) := node(i - 1,j - 1);

          i := i + 2;
        end loop;
        j := j + 2;
      end loop;
    end mesh;

    function spot (i,j : in integer) return integer is

    begin
      return nc * i + j mod nc + 1;
    end spot;

  begin
    open(io, in_file, "prism2.txt");

    -- nc : number of sides of the circle as a polygon
    -- np : number prisms
    scan(io);
    get(io,nc);
    get(io,np);
    skip_line(io);

    -- radius, height
    radius := new terms.fmat1 (0 .. np);
    height := new terms.fmat1 (0 .. np);

    scan(io);
    for i in 0 .. np loop
      get(io,radius(i));
      get(io,height(i));
      skip_line(io);
    end loop;

    -- The rectangle is divided in ni by nj parts.
    -- ni and nj are even numbers.
    scan(io);
    get(io,ni);
    get(io,nj);
    get(io,rho1);
    get(io,rho2);
    get(io,rho3);
    skip_line(io);

    scan(io);
    get(io,area);
    get(io,young);
    get(io,state);
    get(io,value);
    skip_line(io);

    scan(io);
    get(io,stiffness);
    get(io,poisson);

    close(io);

    -- xo, yo, zo
    nn := ((ni + 1) * (nj + 1) + ni * nj - 4) * nc * np + nc * (np + 1);
    xo := new terms.fmat1 (1 .. nn);
    yo := new terms.fmat1 (1 .. nn);
    zo := new terms.fmat1 (1 .. nn);

    -- prism node coordinates
    for i in 0 .. np loop
      for j in 0 .. nc - 1 loop
        glob := spot(i,j);
        angle := (pi + pi) / long_float(nc) * long_float(j);
        xo(glob) := radius(i) * cos(angle);
        yo(glob) := radius(i) * sin(angle);
        zo(glob) := height(i);
      end loop;
    end loop;

    -- elements 1
    ne := nc * np;
    elem1 := new elem1_vec (1 .. ne);

    ne := 0;

    for i in 0 .. np - 1 loop
      if i mod 2 = 0 then
        for j in 0 .. nc - 1 loop
          ne := ne + 1;
          elem1(ne).glob(1) := spot(i,j + 1);
          elem1(ne).glob(2) := spot(i + 1,j);
        end loop;
      else
        for j in 0 .. nc - 1 loop
          ne := ne + 1;
          elem1(ne).glob(1) := spot(i,j);
          elem1(ne).glob(2) := spot(i + 1,j + 1);
        end loop;
      end if;
    end loop;

    -- elements 2
    ne := 4 * ni * nj * nc * np;
    elem2 := new elem2_vec (1 .. ne);

    nn := nc * (np + 1);
    ne := 0;

    for i in 0 .. np - 1 loop
      for j in 0 .. nc - 1 loop
        mesh(ni,nj,rho1,rho2,rho3,spot(i,j),spot(i,j + 1),spot(i + 1,j + 1),spot(i + 1,j));
      end loop;
    end loop;
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist : integer;
    lstol,mntol,mnref      : long_float;
    io                     : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    mntol := 1.0E-06;
    mnref := 1.0;
    maxminor := 30 * nn;
    maxlist := 3;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls             mntol             mnref  maxminor   maxlist");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,mntol,2,6,3);
    put(io,"     ");
    put(io,mnref,2,6,3);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nn,5);
    new_line(io);

    new_line(io);

    for glob in 1 .. nn loop
      put(io,glob,5);
      put(io,"     ");
      put(io,xo(glob),2,6,3);
      put(io,"     ");
      put(io,yo(glob),2,6,3);
      put(io,"     ");
      put(io,zo(glob),2,6,3);
      new_line(io);
    end loop;

    -- elements 1
    new_line(io);
    new_line(io);
    put(io," elem      node      node              area             young");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,elem1'last,5);
    new_line(io);

    new_line(io);
    for e in elem1'range loop
      put(io,e,5);
      for loc in 1 .. 2 loop
        put(io,"     ");
        put(io,elem1(e).glob(loc),5);
      end loop;
      put(io,"     ");
      put(io,area,2,6,3);
      put(io,"     ");
      put(io,young,2,6,3);
      new_line(io);
    end loop;

    -- elements 2
    new_line(io);
    new_line(io);
    put(io," elem      node      node      node         stiffness           poisson");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,elem2'last,5);
    new_line(io);

    new_line(io);
    for e in elem2'range loop
      put(io,e,5);
      for loc in 1 .. 3 loop
        put(io,"     ");
        put(io,elem2(e).glob(loc),5);
      end loop;
      put(io,"     ");
      put(io,stiffness,2,6,3);
      put(io,"     ");
      put(io,poisson,2,6,3);
      new_line(io);
    end loop;

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- set cable element state
    new_line(io);
    new_line(io);
    case state is
      when 1 =>
        put(io," elem     state               cut");
        new_line(io);

      when 2 =>
        put(io," elem     state           tension");
        new_line(io);

      when others =>
        null;
    end case;
    put(io,":");
    new_line(io);
    put(io,elem1'last,5);
    new_line(io);

    new_line(io);
    for e in elem1'range loop
      put(io,e,5);
      put(io,"     ");
      put(io,state,5);
      put(io,"     ");
      put(io,value,2,6,3);
      new_line(io);
    end loop;

    -- loading
    new_line(io);
    new_line(io);
    put(io," node      axis             force");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    glob : integer;
    io   : file_type;

  begin
    create(io, out_file, "prism2.scr");

    -- autocad
    put(io,"osmode 0");
    new_line(io);

    -- elements 2
    put(io,"layer make Elem2 ");
    new_line(io);

    for e in elem2'range loop
      put(io,"3dface ");
      for loc in 1 .. 3 loop
        glob := elem2(e).glob(loc);
        put(io,xo(glob),1,8,3);
        put(io,",");
        put(io,yo(glob),1,8,3);
        put(io,",");
        put(io,zo(glob),1,8,3);
        put(io," ");
      end loop;
      put(io," ");
      new_line(io);
    end loop;

    -- elements 1
    put(io,"layer make Elem1 ");
    new_line(io);

    put(io,"-color");
    new_line(io);
    put(io,"green");
    new_line(io);

    for e in elem1'range loop
      put(io,"3dpoly ");
      for loc in 1 .. 2 loop
        glob := elem1(e).glob(loc);
        put(io,xo(glob),1,8,3);
        put(io,",");
        put(io,yo(glob),1,8,3);
        put(io,",");
        put(io,zo(glob),1,8,3);
        put(io," ");
      end loop;
      new_line(io);
    end loop;

    -- autocad
    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  input;
  autocad;
end prism2;
