with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions,
  terms;

procedure pyramid2 is

  type elem1_rec is
    record
      glob : terms.imat1 (1 .. 2);
    end record;

  type elem1_vec is array (integer range <>) of elem1_rec;

  type elem1_ptr is access elem1_vec;

  type elem2_rec is
    record
      glob : terms.imat1 (1 .. 3);
    end record;

  type elem2_vec is array (integer range <>) of elem2_rec;

  type elem2_ptr is access elem2_vec;

  ns,n1,n2,nn,ne,state : integer;
  radius,height        : long_float;
  area,young,value     : long_float;
  stiffness,poisson    : long_float;
  xo,yo,zo             : terms.fmat1_ptr;
  elem1                : elem1_ptr;
  elem2                : elem2_ptr;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  function node (i,j : in integer) return integer is

    maxj : integer;

  begin
    maxj := i * ns;
    return (i * (i - 1) - n1 * (n1 - 1)) / 2 * ns + 1 + j rem maxj;
  end node;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io,
      ada.numerics,
      ada.numerics.long_elementary_functions;

    e,glob,k  : integer;
    angle     : long_float;
    cost,sint : terms.fmat1_ptr;
    io        : file_type;

  begin
  -- A right pyramidal frustum with a regular polygon basis
  -- The polygon has ns sides
  -- The polygon with radius r1 is divided into ns * n1 equal parts
  -- The polygon with radius r2 is divided into ns * n2 equal parts
  -- The height h is divided in (n2 - n1) parts
  -- r2 > r1 > 0.0
  -- n2 > n1 > 0
  -- The number of diagonals is equal to ns

    open(io, in_file, "pyramid2.txt");

    -- ns, n1, n2, radius, height
    scan(io);
    get(io,ns);
    get(io,n1);
    get(io,n2);
    get(io,radius);
    get(io,height);
    skip_line(io);

    scan(io);
    get(io,area);
    get(io,young);
    get(io,state);
    get(io,value);
    skip_line(io);

    scan(io);
    get(io,stiffness);
    get(io,poisson);
    skip_line(io);

    close(io);

    -- cost, sint
    cost := new terms.fmat1 (0 .. (ns - 1));
    sint := new terms.fmat1 (0 .. (ns - 1));
    angle := 2.0 * pi / long_float(ns);
    for k in 0 .. ns - 1 loop
      cost(k) := cos(angle * long_float(k));
      sint(k) := sin(angle * long_float(k));
    end loop;

    -- xo, yo, zo
    nn := (n2 * (n2 + 1) - n1 * (n1 - 1)) / 2 * ns;
    xo := new terms.fmat1 (1 .. nn);
    yo := new terms.fmat1 (1 .. nn);
    zo := new terms.fmat1 (1 .. nn);

    -- node coordinates
    for i in n1 .. n2 loop
      for j in 0 .. i * ns - 1 loop
        glob := node(i,j);
        k := j / i;
        xo(glob) := radius * (long_float(i - j + k * i) * cost(k) + long_float(j - k * i) * cost((k + 1) rem ns)) / long_float(n2);
        yo(glob) := radius * (long_float(i - j + k * i) * sint(k) + long_float(j - k * i) * sint((k + 1) rem ns)) / long_float(n2);
        zo(glob) := height / long_float(n2 - n1) * long_float(n2 - i);
      end loop;
    end loop;

    -- elements 1
    elem1 := new elem1_vec (1 .. ns);

    for e in elem1'range loop
      elem1(e).glob(1) := node(n1,(e - 1) * n1);
      elem1(e).glob(2) := node(n2,e * n2);
    end loop;

    -- elements 2
    ne := (n2 + n1) * (n2 - n1) * ns;
    elem2 := new elem2_vec (1 .. ne);

    e := 0;

    for i in n1 + 1 .. n2 loop
      for k in 0 .. ns - 1 loop
        for j in 1 .. i loop
          e := e + 1;
          elem2(e).glob(1) := node(i - 1,k * (i - 1) + j - 1);
          elem2(e).glob(2) := node(i,k * i + j - 1);
          elem2(e).glob(3) := node(i,k * i + j);
        end loop;
      end loop;
    end loop;

    for i in n1 .. n2 - 1 loop
      for k in 0 .. ns - 1 loop
        for j in 1 .. i loop
          e := e + 1;
          elem2(e).glob(1) := node(i + 1,k * (i + 1) + j);
          elem2(e).glob(2) := node(i,k * i + j);
          elem2(e).glob(3) := node(i,k * i + j - 1);
        end loop;
      end loop;
    end loop;
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist : integer;
    lstol,mntol,mnref      : long_float;
    io                     : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    mntol := 1.0E-06;
    mnref := 1.0;
    maxminor := 30 * nn;
    maxlist := 3;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls             mntol             mnref  maxminor   maxlist");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,mntol,2,6,3);
    put(io,"     ");
    put(io,mnref,2,6,3);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nn,5);
    new_line(io);

    new_line(io);
    for glob in 1 .. nn loop
      put(io,glob,5);
      put(io,"     ");
      put(io,xo(glob),2,6,3);
      put(io,"     ");
      put(io,yo(glob),2,6,3);
      put(io,"     ");
      put(io,zo(glob),2,6,3);
      new_line(io);
    end loop;

    -- elements 1
    new_line(io);
    new_line(io);
    put(io," elem      node      node              area             young");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,elem1'last,5);
    new_line(io);

    new_line(io);
    for e in elem1'range loop
      put(io,e,5);
      for loc in 1 .. 2 loop
        put(io,"     ");
        put(io,elem1(e).glob(loc),5);
      end loop;
      put(io,"     ");
      put(io,area,2,6,3);
      put(io,"     ");
      put(io,young,2,6,3);
      new_line(io);
    end loop;

    -- elements 2
    new_line(io);
    new_line(io);
    put(io," elem      node      node      node         stiffness           poisson");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,elem2'last,5);
    new_line(io);

    new_line(io);
    for e in elem2'range loop
      put(io,e,5);
      for loc in 1 .. 3 loop
        put(io,"     ");
        put(io,elem2(e).glob(loc),5);
      end loop;
      put(io,"     ");
      put(io,stiffness,2,6,3);
      put(io,"     ");
      put(io,poisson,2,6,3);
      new_line(io);
    end loop;

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- set cable element state
    new_line(io);
    new_line(io);
    case state is
      when 1 =>
        put(io," elem     state               cut");
        new_line(io);

      when 2 =>
        put(io," elem     state           tension");
        new_line(io);

      when others =>
        null;
    end case;
    put(io,":");
    new_line(io);
    put(io,elem1'last,5);
    new_line(io);

    new_line(io);
    for e in elem1'range loop
      put(io,e,5);
      put(io,"     ");
      put(io,state,5);
      put(io,"     ");
      put(io,value,2,6,3);
      new_line(io);
    end loop;

    -- loading
    new_line(io);
    new_line(io);
    put(io," node      axis             force");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    glob : integer;
    io   : file_type;

  begin
    create(io, out_file, "pyramid2.scr");

    -- set running object snap mode to NONE
    put(io,"osmode 0");
    new_line(io);

    -- elements 2
    put(io,"layer make Elem2 ");
    new_line(io);

    for e in elem2'range loop
      put(io,"3dface ");
      for loc in 1 .. 3 loop
        glob := elem2(e).glob(loc);
        put(io,xo(glob),1,8,3);
        put(io,",");
        put(io,yo(glob),1,8,3);
        put(io,",");
        put(io,zo(glob),1,8,3);
        put(io," ");
      end loop;
      put(io," ");
      new_line(io);
    end loop;

    -- elements 1
    put(io,"layer make Elem1 ");
    new_line(io);

    put(io,"-color");
    new_line(io);
    put(io,"green");
    new_line(io);

    for e in elem1'range loop
      put(io,"3dpoly ");
      for loc in 1 .. 2 loop
        glob := elem1(e).glob(loc);
        put(io,xo(glob),1,8,3);
        put(io,",");
        put(io,yo(glob),1,8,3);
        put(io,",");
        put(io,zo(glob),1,8,3);
        put(io," ");
      end loop;
      new_line(io);
    end loop;

    -- autocad
    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  input;
  autocad;
end pyramid2;
